"""An encrypting monitor derived from Sam Rushing's original
encrypting monitor. This variant uses M2Crypto's (i.e., OpenSSL's)
cipher implementations.

Usual disclaimers apply.

Copyright (c) 2000 Ng Pheng Siong. All rights reserved."""

RCS_id='$Id: m2_monitor.py,v 1.1 2000/02/01 15:21:25 ngps Exp ngps $'

from monitor import *
from M2Crypto import EVP, encrypt, decrypt

class m2_secure_encrypted_monitor_channel(secure_monitor_channel):
    "Wrap send() and recv() with an M2Crypto stream cipher (default Blowfish CFB)"

    def __init__ (self, server, conn, addr):
        key = server.password
        self.outgoing = EVP.Cipher(server.cipher, key, iv='s2c', op=encrypt, key_as_bytes=1)
        self.incoming = EVP.Cipher(server.cipher, key, iv='c2s', op=decrypt, key_as_bytes=1)
        secure_monitor_channel.__init__ (self, server, conn, addr)

    def send (self, data):
        return secure_monitor_channel.send (self, self.outgoing.update(data))

    def recv (self, block_size):
        data = secure_monitor_channel.recv (self, block_size)
        if data:
            return self.incoming.update (data)
        else:
            return data

    def close(self):
        try:
            secure_monitor_channel.send(self, self.outgoing.final())
        except:
            pass
        secure_monitor_channel.close(self)


class m2_secure_encrypted_monitor_server(secure_monitor_server):
    channel_class = m2_secure_encrypted_monitor_channel

    def __init__ (self, password, hostname='', port=8023, cipher='bf_cfb'):
        monitor_server.__init__ (self, hostname, port)
        self.password = password
        self.cipher = cipher


# Mostly cribbed from monitor.py.
if __name__ == '__main__':
    import getpass, string, sys
    if '-s' in sys.argv:
        sys.argv.remove ('-s')
        password = getpass.getpass()
    else:
        password = None

    print sys.argv
    if len(sys.argv) > 1:
        port = string.atoi (sys.argv[1])
    else:
        port = 8023

    if password is not None:
        s = m2_secure_encrypted_monitor_server (password, '', port)
    else:
        s = monitor_server ('', port)
    asyncore.loop()


