"""An encrypting monitor client derived from Sam Rushing's original.
This variant uses M2Crypto's (i.e., OpenSSL's) cipher implementations.

Usual disclaimers apply.

Copyright (c) 2000 Ng Pheng Siong. All rights reserved."""

RCS_id='$Id: m2_monitor_client.py,v 1.1 2000/02/01 15:21:36 ngps Exp ngps $'

from monitor_client import *
from M2Crypto import EVP, encrypt, decrypt

class m2_stdin_channel (stdin_channel):
    def handle_read (self):
        data = self.recv(512)
        if not data:
            print '\nclosed.'
            self.sock_channel.push(None)
            self.sock_channel.close()
            try:
                self.close()
            except:
                pass
        data = regsub.gsub ('\n', '\r\n', data)
        self.sock_channel.push (data)


class m2_encrypted_monitor_client (monitor_client):
    "Wrap push() and recv() with an M2Crypto stream cipher (default Blowfish CFB)"

    def init_cipher (self, cipher, key):
        self.outgoing = EVP.Cipher(cipher, key, iv='c2s', op=encrypt, key_as_bytes=1)
        self.incoming = EVP.Cipher(cipher, key, iv='s2c', op=decrypt, key_as_bytes=1)

    def push (self, data):
        if data is not None:
            dd = self.outgoing.update(data)
        else:
            dd = self.outgoing.final()
        return monitor_client.push (self, dd)

    def recv (self, block_size):
        data = monitor_client.recv (self, block_size)
        if data:
            return self.incoming.update (data)
        else:
            return data


if __name__ == '__main__':
    if len(sys.argv) == 1:
        print 'Usage: %s host port' % sys.argv[0]
        sys.exit(0)

    if ('-e' in sys.argv):
        encrypt = 1
        sys.argv.remove ('-e')
    else:
        encrypt = 0

    import getpass
    p = getpass.getpass()
    stdin = m2_stdin_channel (0)
    if len(sys.argv) > 1:
        if encrypt:
            client = m2_encrypted_monitor_client (p, (sys.argv[1], string.atoi (sys.argv[2])))
            client.init_cipher ('bf_cfb', p)
        else:
            client = monitor_client (p, (sys.argv[1], string.atoi (sys.argv[2])))
    else:
        # default to local host, 'standard' port
        client = monitor_client (p)
    stdin.sock_channel = client
    asyncore.loop()

